﻿namespace UT3UserBot;

interface

uses
  System.Collections.Generic,
  System.Linq,
  System.Text,
  System.Threading,
  
  UT3Bots, 
  UT3Bots.UTItems, 
  UT3Bots.Communications;

type
  TMyBot = public class(UTBot)
  private { Field data }
    const verboseLogging: boolean = true;
  private { Event handlers }
    method Events_OnBumped(sender: Object; e: BumpedEventArgs);
    method Events_OnBumpedWall(sender: Object; e: BumpedEventArgs);
    method Events_OnDamaged(sender: Object; e: DamagedEventArgs);
    method Events_OnDied(sender: Object; e: HasDiedEventArgs);
    method Events_OnFoundFall(sender: Object; e: FallEventArgs);
    method Events_OnGotPickup(sender: Object; e: PickupEventArgs);
    method Events_OnHeardNoise(sender: Object; e: HeardSoundEventArgs);
    method Events_OnMatchEnded(sender: Object; e: MatchEndedEventArgs);
    method Events_OnOtherBotDied(sender: Object; e: HasDiedEventArgs);
    method Events_OnPathReceived(sender: Object; e: PathEventArgs);
    method Events_OnReceivedChat(sender: Object; e: ChatEventArgs);
    method Events_OnSeenOtherBot(sender: Object; e: SeenBotEventArgs);
    method Events_OnSpawned(sender: object; e: BotSpawnedEventArgs);
    method Events_OnTaunted(sender: Object; e: TauntedEventArgs);
    method Events_OnWeaponChanged(sender: Object; e: WeaponChangedEventArgs);
    
    method Log(const s: string);
  protected
    method ProcessActions(); override;
  public
    constructor;
  end;
  
implementation

constructor TMyBot;
begin
  inherited constructor(
    '127.0.0.1',        // Server IP Address
    'Delphi Prism Bot', // Bot name
    BotMesh.Harbinger,  // Bot model
    BotColor.Red        // Bot color
  );

  Events.OnBumped += @Events_OnBumped;
  Events.OnBumpedWall += @Events_OnBumpedWall;
  Events.OnDamaged += @Events_OnDamaged;
  Events.OnDied += @Events_OnDied;
  Events.OnFoundFall += @Events_OnFoundFall;
  Events.OnGotPickup += @Events_OnGotPickup;
  Events.OnHeardNoise += @Events_OnHeardNoise;
  Events.OnMatchEnded += @Events_OnMatchEnded;
  Events.OnOtherBotDied += @Events_OnOtherBotDied;
  Events.OnPathReceived += @Events_OnPathReceived;
  Events.OnReceivedChat += @Events_OnReceivedChat;
  Events.OnSeenOtherBot += @Events_OnSeenOtherBot;
  Events.OnSpawned += @Events_OnSpawned;
  Events.OnTaunted += @Events_OnTaunted;
  Events.OnWeaponChanged += @Events_OnWeaponChanged;
end;

//Method to control the bot once it has connected to the server
method TMyBot.ProcessActions();
begin
   // This is your main loop
end;

method TMyBot.Log(const s: string);
begin
  if verboseLogging then
    Console.WriteLine(s);

end;

method TMyBot.Events_OnBumped(sender: Object; e: BumpedEventArgs);
begin
  Log('Bumped: ' + e.Id.ToString);

end;

method TMyBot.Events_OnBumpedWall(sender: Object; e: BumpedEventArgs);
begin
  Log('Bumped Wall: ' + e.Id.ToString);

end;

method TMyBot.Events_OnDamaged(sender: Object; e: DamagedEventArgs);
begin
  Log('Damaged: ' + e.DamageAmount + ' points from ' + e.DamageType);
end;

method TMyBot.Events_OnDied(sender: Object; e: HasDiedEventArgs);
begin
  Log('I died.  ' +  Self.GameState.GetBotNameFromID(e.Killer) + ' killed me.');
  
end;

method TMyBot.Events_OnFoundFall(sender: Object; e: FallEventArgs);
begin
  if e.DidFall then
    Log('Found fall and fell.')
  else
    Log('Just found fall.')

end;

method TMyBot.Events_OnGotPickup(sender: Object; e: PickupEventArgs);
begin
  Log('Got Pickup');

end;

method TMyBot.Events_OnHeardNoise(sender: Object; e: HeardSoundEventArgs);
begin
  Log('Heard Noise');

end;

method TMyBot.Events_OnMatchEnded(sender: Object; e: MatchEndedEventArgs);
begin
  Log('Match Ended');

end;

method TMyBot.Events_OnOtherBotDied(sender: Object; e: HasDiedEventArgs);
begin
  Log('Other Bot Died.  ' + Self.GameState.GetBotNameFromID(e.Killer) + ' killed ' + Self.GameState.GetBotNameFromID(e.Killed));

end;

method TMyBot.Events_OnPathReceived(sender: Object; e: PathEventArgs);
begin
  Log('Path Received');

end;

method TMyBot.Events_OnReceivedChat(sender: Object; e: ChatEventArgs);
begin
  Console.WriteLine('Received Chat from ' + e.FromName + ': ' + e.Message);

end;

method TMyBot.Events_OnSeenOtherBot(sender: Object; e: SeenBotEventArgs);
begin
  Log('Seen Other Bot: ' + e.Name);

end;

method TMyBot.Events_OnSpawned(sender: object; e: BotSpawnedEventArgs);
begin
  Log('Spawned.');
  //Restart your bot's state because we just spawned in the game
  self.Commands.PerformEmote(Emote.BringItOn);
end;

method TMyBot.Events_OnTaunted(sender: Object; e: TauntedEventArgs);
begin
  Log(e.FromName + ' taunted me!');

end;

method TMyBot.Events_OnWeaponChanged(sender: Object; e: WeaponChangedEventArgs);
begin
  Log('Weapon Changed.');

end;

end.